//RGBA cursor support for X11 by Bert Freudenberg <bert@freudenbergs.de>
//
//gcc ioSetCursorARGB.c -o ioSetCursorARGB -lX11 -lXrender -DHAVE_XRENDER

#include<stdlib.h>
#include<stdio.h>
#include<X11/Xlib.h>

#ifdef HAVE_XRENDER
#include<X11/extensions/Xrender.h>
#endif

Display *stDisplay;
Window stWindow;

int ioSetCursorARGB(int cursorBitsIndex, int extentX, int extentY, int offsetX, int offsetY)
{
#if defined(HAVE_XRENDER) && (RENDER_MAJOR>0 || RENDER_MINOR >= 5)
  int eventbase, errorbase;
  if(!XRenderQueryExtension(stDisplay, &eventbase, &errorbase))
    return 0;

  int major = 0;
  int minor = 0; 
  XRenderQueryVersion(stDisplay, &major, &minor);
  if (!(major > 0 || minor >= 5))
    return 0;

  // Okay, we do have at least Render 0.5. Great. Let's make a 
  // Cursor from a Picture from a Pixmap from an Image from our bits
  XImage *image = XCreateImage(stDisplay, DefaultVisual(stDisplay, DefaultScreen(stDisplay)),
			       32, ZPixmap, 0, (char*)cursorBitsIndex, extentX, extentY, 32, 0);
  Pixmap pixmap = XCreatePixmap (stDisplay, DefaultRootWindow(stDisplay), extentX, extentY, 32);
  GC gc = XCreateGC(stDisplay, pixmap, 0, 0);
  XPutImage(stDisplay, pixmap, gc, image, 0, 0, 0, 0, extentX, extentY);
  image->data = 0; //otherwise, XDestroyImage tries to free this
  XRenderPictFormat *pictformat = XRenderFindStandardFormat(stDisplay, PictStandardARGB32);
  Picture picture = XRenderCreatePicture(stDisplay, pixmap, pictformat, 0, 0);
  Cursor cursor = XRenderCreateCursor (stDisplay, picture, offsetX, offsetY);

  // Yay, we got a cursor. Show it, and clean up.
  XDefineCursor(stDisplay, stWindow, cursor);
  XDestroyImage(image);
  XFreeGC(stDisplay, gc);
  XFreeCursor(stDisplay, cursor);
  XRenderFreePicture(stDisplay, picture);
  XFreePixmap(stDisplay, pixmap);
  return 1;
#else
  return 0;
#endif
}

int main(int argc, char *argv[])
{
  stDisplay = XOpenDisplay(":0.0");

  XSynchronize(stDisplay, 1);

  XSetWindowAttributes attr;
  attr.background_pixel = BlackPixel(stDisplay, 0);

  stWindow = XCreateWindow(stDisplay, RootWindow(stDisplay, 0), 0, 0, 100, 100, 0,
				DefaultDepth(stDisplay, 0), InputOutput, DefaultVisual(stDisplay, 0),
				CWBackPixel, &attr);

  XMapWindow(stDisplay, stWindow);


#define R 0xFFFF8080
#define G 0xFF80FF80
#define B 0xFF8080FF
#define H 0X80808080

  unsigned int bits[64] = { 0, 0, R, R, G, G, 0, 0,
			    0, R, R, R, G, G, G, 0,
			    R, R, R, R, G, G, G, G,
			    R, R, R, R, G, G, G, G,
			    B, B, B, B, H, H, H, H,
			    B, B, B, B, H, H, H, H,
			    0, B, B, B, H, H, H, 0,
			    0, 0, B, B, H, H, 0, 0};


  if (!ioSetCursorARGB((int)&bits, 8, 8, 4, 4))
    {
      fprintf(stderr, "Error: ioSetCursorARGB() failed\n");
      exit(1);
    }

  while (1)
    {
      XEvent evt;
      XNextEvent(stDisplay, &evt);
    }
}
